/*
 * RemoveSphereOfAtomsAction.cpp
 *
 *  Created on: May 12, 2010
 *      Author: heber
 */

#include "Actions/WorldAction/RemoveSphereOfAtomsAction.hpp"
#include "atom.hpp"
#include "Descriptors/AtomDescriptor.hpp"
#include "log.hpp"
#include "vector.hpp"
#include "verbose.hpp"
#include "World.hpp"

#include <iostream>
#include <string>

using namespace std;

#include "UIElements/UIFactory.hpp"
#include "UIElements/Dialog.hpp"
#include "Actions/MapOfActions.hpp"

const char WorldRemoveSphereOfAtomsAction::NAME[] = "remove-sphere";

WorldRemoveSphereOfAtomsAction::WorldRemoveSphereOfAtomsAction() :
  Action(NAME)
{}

WorldRemoveSphereOfAtomsAction::~WorldRemoveSphereOfAtomsAction()
{}

Action::state_ptr WorldRemoveSphereOfAtomsAction::performCall() {
  Dialog *dialog = UIFactory::getInstance().makeDialog();
  double radius = 0.;
  atom *first = NULL;

  dialog->queryDouble(NAME, &radius, MapOfActions::getInstance().getDescription(NAME));
  dialog->queryAtom("atom-by-id", &first, MapOfActions::getInstance().getDescription("atom-by-id"));

  if(dialog->display()) {
    delete dialog;
    DoLog(1) && (Log() << Verbose(1) << "Removing atoms around " << first->nr << " with radius " << radius << "." << endl);
    vector<atom*> AllAtoms = World::getInstance().getAllAtoms();
    vector<atom*>::iterator AtomAdvancer = AllAtoms.begin();
    for (vector<atom*>::iterator AtomRunner = AtomAdvancer; AtomRunner != AllAtoms.end(); ) {
      ++AtomAdvancer;
      if (first != *AtomRunner) // dont't destroy reference ...
        if (first->x.DistanceSquared((*AtomRunner)->x) > radius*radius) // distance to first above radius ...
          World::getInstance().destroyAtom(*AtomRunner);
    }
    World::getInstance().destroyAtom(first);
    return Action::success;
  } else {
    delete dialog;
    return Action::failure;
  }

}

Action::state_ptr WorldRemoveSphereOfAtomsAction::performUndo(Action::state_ptr _state) {
//  ParserLoadXyzState *state = assert_cast<ParserLoadXyzState*>(_state.get());

  return Action::failure;
//  string newName = state->mol->getName();
//  state->mol->setName(state->lastName);
//
//  return Action::state_ptr(new ParserLoadXyzState(state->mol,newName));
}

Action::state_ptr WorldRemoveSphereOfAtomsAction::performRedo(Action::state_ptr _state){
  return Action::failure;
}

bool WorldRemoveSphereOfAtomsAction::canUndo() {
  return false;
}

bool WorldRemoveSphereOfAtomsAction::shouldUndo() {
  return false;
}

const string WorldRemoveSphereOfAtomsAction::getName() {
  return NAME;
}
